# Generated by Django 3.2.8 on 2022-02-11 14:49

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_ddns', '0007_zone_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='server',
            name='server_port',
            field=models.PositiveIntegerField(default=53, validators=[django.core.validators.MinValueValidator(53), django.core.validators.MaxValueValidator(65535)]),
        ),
    ]
